package ittp;

import ittp.protocol.request.Request;
import ittp.protocol.response.Response;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;
import java.util.ResourceBundle;
import java.rmi.*;
import java.rmi.server.*;
import java.security.*;
import java.net.SocketPermission;
import java.util.Enumeration;

/**
 * This class allow to manage the server from a GUI.
 * @author Frederic Bidon and Mathieu Texier
 */
public class Control extends JDialog {
    /**
     * Title of the window
     */
    final static String SERVER_TITLE = "ITTP Server Control";
    /**
     * Remote object to witch we want to access
     */
    public RemoteControl parameters;
    /**
     * the control panel instance
     */
    ControlPanel controlPanel;
    
    /**Constructor of the class*/
    public Control (Frame parent, boolean modal) {
        super (parent, modal);
        
        try {
            String rmiServer = "rmi://localhost/IttpServer";
            String Value = JOptionPane.showInputDialog("Please enter The address of the rmi Server. \n" +
            "(leave the field empty to choose the localhost)\n" + 
            "(cancel to leave the application)"); 
            if (Value == null)
                System.exit(0);
            if (Value.length () > 0)
                rmiServer = Value;
            
            parameters = (RemoteControl) Naming.lookup (rmiServer);
        }  catch (Exception e) {
            alert(e);
        }
        
        setResizable (false);
        setTitle (SERVER_TITLE);
        String laf = UIManager.getSystemLookAndFeelClassName ();
        try {
            UIManager.setLookAndFeel (laf);
        } catch (UnsupportedLookAndFeelException exc) {
            System.err.println ("Warning: UnsupportedLookAndFeel: " + laf);
        } catch (Exception exc) {
            System.err.println ("Error loading " + laf + ": " + exc);
        }
        
        addWindowListener (new WindowAdapter () {
            public void windowClosing (WindowEvent evt) {
                System.exit (0);
            }
        });
        
        controlPanel = new ControlPanel ();
        getContentPane ().add (controlPanel, BorderLayout.CENTER);
        pack ();
        setVisible (true);
    }
    /**
     * Main entry of the management server.
     * Create a new Thread Refresh that will manage the server.
     * @param args the command line arguments
     */
    public static void main (String args[]) {
        Control control = new Control (new JFrame (), false);
        Refresh refresh = control.new Refresh ();
        Thread t = new Thread (refresh);
        t.start ();
    }
    
    /**
     * Message box that display to alert the user
     * @param exception Exception to be displayed
     */
    static public void alert (Exception exception) {
        JOptionPane.showMessageDialog (null, exception.getMessage (), "Error", JOptionPane.ERROR_MESSAGE);
    }
    
    /**
     * Message box that display to alert the user
     * @param info information to be displayed
     */
    static public void alert (String info) {
        JOptionPane.showMessageDialog (null, info, "Information", JOptionPane.INFORMATION_MESSAGE);
    }
    
    /**
     * Thread executing the management server
     */
    class Refresh  implements Runnable {
        private int delay = 100;
        /**
         * Refresh the value every 100 ms
         */
        public void run () {
            while (true) {
                try {
                    Thread.sleep (delay);
                    controlPanel.setActive (parameters.getNumberOfActiveConection ());
                } catch (InterruptedException e) {
                } catch (RemoteException e) {
                    alert(e);
                }
            }
        }
    }
    
    /**
     * inner class of Control that extends JPanel
     * Build the interface graphique for the management server
     */
    class ControlPanel extends JPanel {
        
        /**
         * Creates new form Control panel
         */
        public ControlPanel () {
            titleLabel = new JLabel ();
            titleLabel.setFont (new Font ("MS Sans Serif", 1, 18));
            titleLabel.setHorizontalAlignment (SwingConstants.CENTER);
            titleLabel.setText (Control.SERVER_TITLE);
            add (titleLabel);
            
            StartBtn = new JToggleButton ();
            StartBtn.setText ("Start");
            try {
                StartBtn.setSelected ((parameters.getServerState () == Parameters.START)? false : true);
            } catch (RemoteException e) {System.out.println (e);}
            StartBtn.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    StartBtnActionPerformed (evt);
                }
            });
            add (StartBtn);
            
            nActive = new JLabel ();
            try {
                nActive.setText (Integer.toString (parameters.getNumberOfRequest ()));
            } catch (RemoteException e) {System.out.println (e);}
            add (nActive);
            
            ActiveLabel = new JLabel ();
            ActiveLabel.setText ("Connection Active /");
            add (ActiveLabel);
            
            
            
            totalLabel = new JLabel ();
            totalLabel.setText ("total");
            add (totalLabel);
            
            timeOutLabel = new JLabel ();
            timeOutLabel.setText ("Timeout");
            add (timeOutLabel);
            
            shutDownBtn = new JButton ();
            shutDownBtn.setText ("Shut down");
            shutDownBtn.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    shutDownBtnActionPerformed (evt);
                }
            });
            add (shutDownBtn);
            
            secLabel = new JLabel ();
            secLabel.setText ("s");
            add (secLabel);
            
            closeBtn = new JButton ();
            closeBtn.setText ("Close");
            add (StartBtn);
            closeBtn.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    System.exit (0);
                }
            });
            add (closeBtn);
            
            nTotal = new JSpinner ();
            nTotal.setValue (new Integer (Parameters.DEFAULT_CONNECTION));
            nTotal.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    nTotalActionPerformed (evt);
                }
            });
            add (nTotal);
            
            nTimeOut = new JSpinner ();
            nTimeOut.setValue (new Integer (Parameters.DEFAULT_TIMEOUT));
            nTimeOut.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    nTimeOutActionPerformed (evt);
                }
            });
            add (nTimeOut);
            
            headerSeparator = new JSeparator ();
            headerSeparator.setBorder (new SoftBevelBorder (BevelBorder.RAISED));
            add (headerSeparator);
            
            getCheckBox = new JCheckBox ();
            getCheckBox.setText ("GET");
            getCheckBox.setSelected (true);
            getCheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedMethod (Request.GET, getCheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (getCheckBox);
            
            headCheckBox = new JCheckBox ();
            headCheckBox.setText ("HEAD");
            headCheckBox.setSelected (true);
            headCheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedMethod (Request.HEAD, headCheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (headCheckBox);
            
            optionsCheckBox = new JCheckBox ();
            optionsCheckBox.setText ("OPTIONS");
            optionsCheckBox.setSelected (true);
            optionsCheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedMethod (Request.OPTIONS, optionsCheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (optionsCheckBox);
            
            traceCheckBox = new JCheckBox ();
            traceCheckBox.setText ("TRACE");
            traceCheckBox.setSelected (true);
            traceCheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedMethod (Request.TRACE, traceCheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (traceCheckBox);
            
            AllowCheckBox = new JCheckBox ();
            AllowCheckBox.setText ("Allow");
            AllowCheckBox.setSelected (true);
            AllowCheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedHeader (Response.ALLOW, AllowCheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (AllowCheckBox);
            
            contentMD5CheckBox = new JCheckBox ();
            contentMD5CheckBox.setText ("Content-MD5");
            contentMD5CheckBox.setSelected (true);
            contentMD5CheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedHeader (Response.CONTENT_MD5, contentMD5CheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (contentMD5CheckBox);
            
            contentTypeCheckBox = new JCheckBox ();
            contentTypeCheckBox.setText ("Content-Type");
            contentTypeCheckBox.setSelected (true);
            contentTypeCheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedHeader (Response.CONTENT_TYPE, contentTypeCheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (contentTypeCheckBox);
            
            serverCheckBox = new JCheckBox ();
            serverCheckBox.setText ("Server");
            serverCheckBox.setSelected (true);
            serverCheckBox.addChangeListener (new ChangeListener () {
                public void stateChanged (ChangeEvent evt) {
                    try {
                        parameters.setAllowedHeader (Response.SERVER, serverCheckBox.isSelected ());
                    } catch (RemoteException e) {System.out.println (e);}
                }
            });
            add (serverCheckBox);
            
            methodSeparator = new JSeparator ();
            methodSeparator.setBorder (new SoftBevelBorder (BevelBorder.RAISED));
            add (methodSeparator);
            
            statBtn = new JButton ();
            statBtn.setText ("Statistic");
            statBtn.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    statBtnActionPerformed (evt);
                }
            });
            add (statBtn);
        }
        
        /**
         * Causes this container to lay out its components.
         */
        public void doLayout () {
            titleLabel.setBounds (0, 0, 300, 25);
            StartBtn.setBounds (20, 40, 60, 25);
            ActiveLabel.setBounds (40, 80, 100, 25);
            nActive.setBounds (20, 80, 20, 25);
            totalLabel.setBounds (190, 80, 80, 25);
            timeOutLabel.setBounds (20, 100, 50, 25);
            shutDownBtn.setBounds (90, 40, 100, 25);
            nTimeOut.setBounds (70, 100, 40, 20);
            nTotal.setBounds (150, 80, 40, 20);
            secLabel.setBounds (110, 100, 40, 25);
            headerSeparator.setBounds (20, 130, 120, 100);
            getCheckBox.setBounds (160, 140, 100, 25);
            headCheckBox.setBounds (160, 160, 100, 25);
            optionsCheckBox.setBounds (160, 180, 100, 25);
            traceCheckBox.setBounds (160, 200, 100, 25);
            AllowCheckBox.setBounds (20, 140, 100, 25);
            contentMD5CheckBox.setBounds (20, 160, 100, 25);
            contentTypeCheckBox.setBounds (20, 180, 100, 25);
            serverCheckBox.setBounds (20, 200, 100, 25);
            methodSeparator.setBounds (150, 130, 120, 100);
            statBtn.setBounds (20, 250, 100, 25);
            closeBtn.setBounds (210, 250, 60, 25);
        }
        
        /**
         * Minimum dimension of the statistic windows
         * @return an instance of Dimension that represents the minimum size of this container.
         */
        public Dimension getMinimumSize () {
            return new Dimension (300,290);
        }
        
        /**
         * Prefered dimension of the statistic windows
         * @return an instance of Dimension that represents the preferred size of this container.
         */
        public Dimension getPreferredSize () {
            return new Dimension (300,290);
        }
        
        public void shutDownBtnActionPerformed (ActionEvent evt) {
            try {
                parameters.setServerState (Parameters.SHUT_DOWN);
            } catch (RemoteException e) {System.out.println (e);}
        }
        
        public void StartBtnActionPerformed (ActionEvent evt) {
            try {
                parameters.setServerState (StartBtn.isSelected ()? Parameters.START:Parameters.STOP);
            } catch (RemoteException e) {System.out.println (e);}
        }
        
        public void nTotalActionPerformed (ChangeEvent evt) {
            
            try {
                Integer num = (Integer) nTotal.getValue ();
                int nVal = parameters.setConnectionMax (num.intValue ());
                nTotal.setValue (new Integer (nVal));
            } catch (RemoteException e) {
                System.out.println (e);
            } catch (NumberFormatException e) {
                Control.alert (e);
            }
        }
        
        public void nTimeOutActionPerformed (ChangeEvent evt) {
            try {
                Integer num = (Integer) nTimeOut.getValue ();
                int nVal = parameters.setTimeOut (num.intValue ());
                nTimeOut.setValue (new Integer (nVal));
            } catch (RemoteException e) {
                System.out.println (e);
            } catch (NumberFormatException e) {
                Control.alert (e);
            }
        }
        
        public void statBtnActionPerformed (ActionEvent evt) {
            Statistic stat = new Statistic (new JFrame (), false, parameters);
            Statistic.Refresh refresh = stat.new Refresh ();
            Thread t = new Thread (refresh);
            t.start ();
        }
        
        public void setActive (int active) {
            nActive.setText (Integer.toString (active));
        }
        
        private JToggleButton StartBtn;
        private JButton closeBtn;
        private JButton shutDownBtn;
        private JButton statBtn;
        
        private JCheckBox contentMD5CheckBox;
        private JCheckBox contentTypeCheckBox;
        private JCheckBox getCheckBox;
        private JCheckBox headCheckBox;
        private JCheckBox AllowCheckBox;
        private JCheckBox serverCheckBox;
        private JCheckBox optionsCheckBox;
        private JCheckBox traceCheckBox;
        
        private JSpinner nTimeOut;
        private JSpinner nTotal;
        
        private JSeparator headerSeparator;
        private JSeparator methodSeparator;
        
        private JLabel nActive;
        private JLabel totalLabel;
        
        private JLabel ActiveLabel;
        private JLabel secLabel;
        private JLabel timeOutLabel;
        private JLabel titleLabel;
        
    }
}

class Statistic extends JDialog {
    /**
     * The title of the statistic windows
     */
    final static String STAT_TITLE = "Statistic";
    public RemoteControl parameters;
    
    /**
     * Constructor of the statistic windows
     */
    public Statistic (Frame parent, boolean modal, RemoteControl parameters) {
        super (parent, modal);
        this.parameters = parameters;
        
        setResizable (false);
        setTitle (STAT_TITLE);
        String laf = UIManager.getSystemLookAndFeelClassName ();
        try {
            UIManager.setLookAndFeel (laf);
        } catch (UnsupportedLookAndFeelException exc) {
            System.err.println ("Warning: UnsupportedLookAndFeel: " + laf);
        } catch (Exception exc) {
            System.err.println ("Error loading " + laf + ": " + exc);
        }
        
        addWindowListener (new WindowAdapter () {
            public void windowClosing (WindowEvent evt) {
                hide ();
            }
        });
        
        getContentPane ().add (new StatPanel (), BorderLayout.CENTER);
        pack ();
        setVisible (true);
    }
    
    class Refresh  implements Runnable {
        private int delay = 100;
        /**
         * refresh the statistic
         */
        public void run () {
            while (true) {
                try {
                    Thread.sleep (delay);
                    countError.setText (Integer.toString (parameters.getNumberOfError ()) + " Errors");
                    countRequest.setText (Integer.toString (parameters.getNumberOfRequest ()) + " Requests");
                    countOctet.setText (Integer.toString (parameters.getNumberOfOctet ()) + " Octets transfered");
                    countConnect.setText (Integer.toString (parameters.getNumberOfConnect ()) + " Connections");
                } catch (InterruptedException e) {
                } catch (RemoteException e) {
                    System.out.println (e);
                }
            }
        }
    }
    
    /**
     * instance to the main panel
     */
    class StatPanel extends JPanel {
        
        /** Creates new form Co */
        public StatPanel () {
            countError = new JLabel ();
            add (countError);
            
            countRequest = new JLabel ();
            add (countRequest);
            
            countOctet = new JLabel ();
            add (countOctet);
            
            countConnect = new JLabel ();
            add (countConnect);
            
            resetBtn = new JButton ();
            resetBtn.setText ("reset");
            resetBtn.addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    resetBtnActionPerformed (evt);
                }
            });
            add (resetBtn);
        }
        
        /**
         * Causes this container to lay out its components.
         */
        public void doLayout () {
            countError.setBounds (20, 20, 150, 25);
            countRequest.setBounds (20, 40, 150, 25);
            countOctet.setBounds (20, 60, 150, 25);
            countConnect.setBounds (20, 80, 150, 25);
            resetBtn.setBounds (30, 120, 110, 25);
        }
        /**
         * Minimun dimension of the statistic windows
         * @return an instance of Dimension that represents the minimum size of this container.
         */
        public Dimension getMinimumSize () {
            return new Dimension (170,160);
        }
        
        /**
         * Prefered dimension of the statistic windows
         * @return an instance of Dimension that represents the preferred size of this container.
         */
        public Dimension getPreferredSize () {
            return new Dimension (170,160);
        }
        
        public void resetBtnActionPerformed (ActionEvent evt) {
            try {
                parameters.resetRecord ();
            } catch (RemoteException e) {System.out.println (e);}
        }
    }
    private JLabel countError;
    private JLabel countRequest;
    private JLabel countOctet;
    private JLabel countConnect;
    private JButton resetBtn;
}
