package ittp.protocol.request.headers;

import ittp.protocol.request.Request;
import ittp.protocol.Protocol;
import ittp.ittpException;
import ittp.protocol.response.Response;
import java.text.ParsePosition;
import java.text.FieldPosition;
import java.text.DateFormat;

/**
 * This class analyse the header Last-Modified send by the client.
 * @author Frederic Bidon and Mathieu Texier
 */
public class ModifiedSince extends Request.Headers {
    private java.util.Date dateModifiedSince;
    private boolean used = false;
    private Request parent;
    
    /**Constructor of the class*/
    public ModifiedSince (Request parent) {
        parent.super ();
        this.parent = parent;
    }
    
    /**
     * Mutators for the String dateText.
     * Check if the format of the date is correct.
     * @param dateText String that contain the date given by the client.
     * @throws ittpException if the headers If-Modified-Since already received or if the format of the date is incorrect.
     */
    public void setModifiedSince (String dateText) throws ittpException {
        if (used)
            throw new ittpException ("headers If-Modified-Since already received", Response.Code.ITTP_BAD_REQUEST);
        ParsePosition parse = new ParsePosition (0);
        dateModifiedSince = Protocol.dateFormat.parse (dateText, parse);
        int errorIndex = parse.getErrorIndex ();
        if (errorIndex != -1)
            throw new ittpException ("Wrong date Format " + dateText + " at character n=" + errorIndex, Response.Code.ITTP_BAD_REQUEST);
        used = true;
    }
    
    /**
     * Compare the date given by the client with the current date of the file.
     * @param date long that contain the current date of the file.
     * @return true if the current date is inferior to the date given by the client.
     */
    public boolean isModifiedSince (long date) {
        java.util.Date fileDate = new java.util.Date (date);
        if (dateModifiedSince.before (fileDate))
            return true;
        else
            return false;
    }
    
    /**
     * Convert to a string the date of the last modification.
     * @return a string containing the date of the last modification of the file.
     */
    public String getModifiedSince () {
        StringBuffer strBuf = new StringBuffer ();
        strBuf = Protocol.dateFormat.format (dateModifiedSince, strBuf ,new FieldPosition (0));
        return strBuf.toString ();
    }
    
    /**
     * Convert to a string the headers last-modified send by the client.
     * @return a string containing the response to this headers if it is used.
     */
    public String toString () {
        if (used)
            return "If-Modified-Since:" + getModifiedSince () + "\n";
        else return "";
    }
    
    /**
     * Accessors for the boolean used.
     * @return used: boolean that determine if this headers is used.
     */
    public boolean isUsed () {
        return used;
    }
}
